// #![warn(clippy::all, clippy::pedantic, clippy::restriction)]
mod colorscheme;
mod config;
mod document;
mod editor;
mod events;
mod row;
mod statusbar;
mod terminal;
mod helpmenu;
mod renderer;

use renderer::Renderer;
use editor::Editor;
pub use colorscheme::Colorscheme;
pub use config::Config;
pub use document::Document;
pub use editor::Position;
pub use events::Events;
pub use row::Row;
pub use statusbar::Statusbar;
pub use terminal::Terminal;


use std::error::Error;
use tokio::sync::mpsc::{unbounded_channel};
use tokio::select;


pub struct SharedInfo {
    cpos : Position,
    doc : Document,
    stbar : Statusbar
}


#[tokio::main]
async fn main() {


    let (st, rt) = unbounded_channel();
    let (ist, irt) = unbounded_channel();


    let mut edit = Editor::default(irt, st.clone());

    let shared = edit.shared.clone();


    let mut render = Renderer::default(st, ist, rt, shared);

    // TODO: ERROR CARCHING INSIDE TOKIO THREADS
    //  tokio::task::JoinHandle<std::result::Result<(), Box<dyn Error + Send + Sync>>>


    let t1 : tokio::task::JoinHandle<std::result::Result<(), Box<dyn Error + Send + Sync>>> = tokio::spawn(async move {
        render.run().await?;
        Ok(())
    });

    let t2 : tokio::task::JoinHandle<std::result::Result<(), Box<dyn Error + Send + Sync>>> = tokio::spawn(async move {
        edit.run().await?;
        Ok(())
    });



    // sleep(Duration::from_secs(10)).await;
    select!(
        k = t1 => {
            match k {
                Err(e) => println!("Catastrophic error: \n{:?}", e), // TODO: crash handling
                Ok(_) => return
            }
        }

        k = t2 => {
            match k {
                Err(e) => println!("Catastrophic error: \n{:?}", e), // TODO: crash handling
                Ok(_) => return
            }
        }
    )

}
