use crate::Document;
use crate::Events;
use crate::Position;

use std::error::Error;
use std::sync::{Arc, RwLock};
use std::time::Duration;

use chrono::Local;

use tokio::sync::mpsc::{unbounded_channel, UnboundedSender};
use tokio::time::timeout;

pub struct Statusbar {
    statussend: UnboundedSender<()>,

    pub prompting: bool,
    sharedstatus: Arc<RwLock<String>>,
    pub status_stat: String,
    pub status2_stat: String,

    _statusthread: tokio::task::JoinHandle<()>,
    // _timethread: std::thread::JoinHandle<()>
}

impl Statusbar {
    pub fn new(tsd: UnboundedSender<Events>) -> Self {
        let (st, mut rt) = unbounded_channel();

        let sharedstatus = Arc::new(RwLock::new(String::default()));
        let status_stat = String::from("");
        let status2_stat = String::from("");
        let statusclone = sharedstatus.clone();

        let tsdc = tsd.clone();

        let t = tokio::spawn(async move {
            loop {
                rt.recv().await;

                loop {
                    match timeout(Duration::from_secs(3), rt.recv()).await {
                        Err(_) => {
                            let mut w = statusclone.write().unwrap();
                            *w = String::from("");
                            tsdc.clone().send(Events::Refresh);
                            break;
                        }
                        Ok(_) => {}
                    }
                }
            }
        });

        Self {
            statussend: st,
            prompting: false,

            sharedstatus,
            status_stat,
            status2_stat,

            _statusthread: t,
        }
    }

    pub fn change_status(&mut self, message: &str) -> Result<(), Box<dyn Error + Send + Sync>> {
        let mut s = self.sharedstatus.write().unwrap();
        *s = String::from(message);

        self.statussend.send(())?; // change error type
        Ok(())
    }

    pub fn status(&self) -> String {
        if self.prompting {
            self.status_stat.clone()
        } else {
            let s = &*self.sharedstatus.read().unwrap();
            s.clone()
        }
    }

    pub fn status2(&self, doc: &Document, pos: &Position, width: usize) -> String {
        if self.prompting {
            self.status2_stat.clone()
        } else {
            let filename = doc.filename.clone().unwrap_or(String::from("No filename"));

            let modified = if doc.dirty { "+" } else { "" };

            let mut status2 = format!(" {} {} | {}, {} |", filename, modified, pos.x, pos.y);

            let date = format!("{}  ", Local::now().format("%H:%M").to_string());

            if width > status2.len() {
                status2.push_str(&" ".repeat(width - status2.len() - date.len()))
            } else {
                status2.truncate(width);
            }

            status2.push_str(&date);

            status2
        }
    }
}
